/* Copyright (C) 1996 Park Myeong Seok <pms@romance.kaist.ac.kr>.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#include <string.h>
#include "muttlib.h"
#include "rfc_gen.h"

/* 
** This is a good place to implement nation specific RFCs.
** Currently only supports RFC-1557(for Korean).
**
** Return your header encoding method.
**   Q_HEADER = 0 - RFC-1522 default  (Q-encoded header)
**   B_HEADER = 1 - RFC-1522 optional (B-encoded header)
*/
int rfc_own_header_coding(void)
{
   if(CompatKorean(Charset)) return B_HEADER;
   /* Return header encoding method of your RFC's */
   /* I know this is ugly coding.. */
   return Q_HEADER;
}

/* 
** This is a good place to implement nation specific RFCs.
** Currently only supports RFC-1557(for Korean).
**
** Return 1, if decoding was performed.
*/ 
int mutt_decode_own_rfc(STATE *s, long len, char *charset)
{
   if(!charset) charset = "us-ascii";
   if(CompatKorean(charset) || !strcasecmp(charset, "us-ascii")) {
      DecodeISO2022KR(s, len);
      return 1;
   }
   /* Insert here your nation's RFC decoding routine */
   /* I know this is ugly coding.. */
/*
** This is not a desired method, but we should consider the fault of users.
** They frequently do not set their native charset as a content charset.
*/ 
   return DecodeISO2022KR(s, len);
}

/* 
** This is a good place to implement nation specific RFCs.
** Currently only supports RFC-1557(for Korean).
**
** Return 1, if encoding was performed.
*/ 
int mutt_encode_own_rfc(FILE *fpin, FILE *f)
{
   if(CompatKorean(Charset)) {
      EncodeISO2022KR(fpin, f);
      return 1;
   }
   /* Insert here your nation's RFC encoding routine */
   /* I know this is ugly coding.. */
   return 0;
}

/* 
** This is a good place to implement nation specific RFCs.
** Currently only supports RFC-1557(for Korean).
**
** Return your charset.
*/
char* rfc_own_charset(int allow_8bit, int is_8bit)
{
   if(CompatKorean(Charset)) {
      if(!is_8bit) return "us-ascii";
      if(allow_8bit) return "euc-kr";
      else return "iso-2022-kr";
   }
   /* Set here your nation's Charset */
   /* I know this is ugly coding.. */
   return (Charset);
}

/* 
** This is a good place to implement nation specific RFCs.
** Currently only supports RFC-1557(for Korean).
**
** Return 1, if your Charset is encoded with different method as compared
** with one specified in RFC-1522.
*/ 
int rfc_own_content_encoding(void)
{
   if(CompatKorean(Charset)) return 1;
   /* Insert here your nation's implementation */
   /* I know this is ugly coding.. */
   return 0;
}

/* 
** This is a good place to implement nation specific RFCs.
** Currently only supports RFC-1557(for Korean).
**   
** Return 1 if your Charset and other charsets should be compatible.
*/
int rfc_own_compat_charset(char* charset)
{
   if(!charset) charset = "us-ascii";
   if(!strcasecmp(charset, "us-ascii") || CompatKorean(charset))
      return 1;
   /* Insert here your nation's Charset compatibility */
   /* I know this is ugly coding.. */
   return 0;
}
